from ..object_classes.wireframe_object import WireframeObject
import numpy as np


class Cuboid(WireframeObject):
    def __init__(
        self,
        bottom_corner=np.array([0, 0, 0]),
        top_corner=np.array([1, 1, 1]),
        position=np.array([0.0, 0.0, 0.0]),
        color=(0, 0, 0),
    ):
        self.top_corner = top_corner
        self.bottom_corner = bottom_corner

        vertices = np.array(
            [
                np.array([bottom_corner[0], bottom_corner[1], bottom_corner[2]]),
                np.array([top_corner[0], bottom_corner[1], bottom_corner[2]]),
                np.array([top_corner[0], top_corner[1], bottom_corner[2]]),
                np.array([bottom_corner[0], top_corner[1], bottom_corner[2]]),
                np.array([bottom_corner[0], bottom_corner[1], top_corner[2]]),
                np.array([top_corner[0], bottom_corner[1], top_corner[2]]),
                np.array([top_corner[0], top_corner[1], top_corner[2]]),
                np.array([bottom_corner[0], top_corner[1], top_corner[2]]),
            ]
        )

        lines = [
            (0, 1),
            (1, 2),
            (2, 3),
            (3, 0),
            (4, 5),
            (5, 6),
            (6, 7),
            (7, 4),
            (0, 4),
            (1, 5),
            (2, 6),
            (3, 7),
        ]

        super().__init__(vertices, lines, position, color)

    def check_collision(self, point):
        # use vertices not corners to check collision
        return (
            self.vertices[0][0] <= point[0] <= self.vertices[1][0]
            and self.vertices[0][1] <= point[1] <= self.vertices[2][1]
            and self.vertices[0][2] <= point[2] <= self.vertices[4][2]
        )



from ..object_classes.flat_faces_object import FlatFacesObject
import numpy as np


class Vector(FlatFacesObject):
    def __init__(
        self,
        start_point=np.array([0.0, 0.0, 0.0]),
        vector_components=np.array([1.0, 1.0, 1.0]),
        color=(0, 0, 0),
        thickness=0.02,
    ):
        """
        Vector object
        :param start_point: the start point of the vector
        :param vector_components: the direction and length of the vector
        :param color: the color of the vector
        :param thickness: the thickness of the vector
        """
        start_point = 0.5 * np.array(start_point)

        self.start_point = start_point
        self.vector_components = vector_components
        self.end_point = start_point + vector_components
        self.thickness = thickness
        self.color = color

        direction = vector_components / np.linalg.norm(vector_components)
        vertices, faces = self.create_faces(
            self.start_point, self.end_point, direction, thickness, color
        )

        super().__init__(
            vertices=vertices,
            faces=faces,
            color=color,
            compile_verts=True,
            position=start_point,
            move_to_zero=False,
        )

    def update_vector(self, start_point=None, vector_components=None):
        """
        Update the vector
        :param start_point: the start point of the vector
        :param vector_components: the direction and length of the vector
        """
        if start_point is not None:
            start_point = 0.5 * np.array(start_point)
            self.start_point = start_point

        if vector_components is not None:
            self.vector_components = vector_components

        self.end_point = self.start_point + self.vector_components

        direction = vector_components / np.linalg.norm(vector_components)
        vertices, faces = self.create_faces(
            self.start_point, self.end_point, direction, self.thickness, self.color
        )

        self.faces = faces
        self.vertices = vertices


    def create_faces(self, start, end, direction, thickness, color):
        # Define the vertices and faces
        perpendicular_vector = np.cross(direction, np.array([1, 0, 0]))
        if (
            np.linalg.norm(perpendicular_vector) < 1e-6
        ):  # direction was collinear with x-axis
            perpendicular_vector = np.cross(direction, np.array([0, 1, 0]))

        perpendicular_vector = (
            perpendicular_vector / np.linalg.norm(perpendicular_vector) * thickness
        )
        perpendicular_vector2 = np.cross(direction, perpendicular_vector)
        perpendicular_vector2 = (
            perpendicular_vector2 / np.linalg.norm(perpendicular_vector2) * thickness
        )

        vertices = np.array(
            [
                start + perpendicular_vector,
                start - perpendicular_vector,
                start + perpendicular_vector2,
                start - perpendicular_vector2,
                end + perpendicular_vector,
                end - perpendicular_vector,
                end + perpendicular_vector2,
                end - perpendicular_vector2,
            ]
        )

        faces = [
            ([0, 1, 5, 4], color),  # Face 1
            ([2, 3, 7, 6], color),  # Face 2
        ]

        return vertices.tolist(), faces



import numpy as np
from ..object_classes.flat_faces_object import FlatFacesObject


def generate_sphere_faces(radius, resolution):
    theta = np.linspace(0, np.pi, resolution)
    phi = np.linspace(0, 2 * np.pi, resolution)
    THETA, PHI = np.meshgrid(theta, phi)

    x = radius * np.sin(THETA) * np.cos(PHI)
    y = radius * np.sin(THETA) * np.sin(PHI)
    z = radius * np.cos(THETA)

    vertices = np.stack((x.flatten(), y.flatten(), z.flatten()), axis=-1)

    faces = []
    for i in range(resolution - 1):
        for j in range(resolution - 1):
            index = i * resolution + j
            faces.append(
                (
                    [index, index + 1, index + resolution + 1, index + resolution],
                    (255, 111, 100),
                )
            )

    # Connect the last row with the first one to close the sphere
    for j in range(resolution - 1):
        index = (resolution - 1) * resolution + j
        faces.append(([index, index + 1, j + 1, j], (255, 111, 100)))

    return vertices, faces


class Fsphere(FlatFacesObject):
    def __init__(
        self,
        radius=1.0,
        position=np.array([0.0, 0.0, 0.0]),
        color=(0, 0, 0),
        resolution=20,
    ):
        vertices, faces = generate_sphere_faces(radius, resolution)
        super().__init__(vertices, faces, position, color)
